@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 3465 2008-01-16 00:55:26Z joanied $"
-__version__   = "$Revision: 3465 $"
-__date__      = "$Date: 2008-01-15 19:55:26 -0500 (Tue, 15 Jan 2008) $"
+__id__        = "$Id: speechgenerator.py 3638 2008-02-25 21:50:59Z joanied $"
+__version__   = "$Revision: 3638 $"
+__date__      = "$Date: 2008-02-25 16:50:59 -0500 (Mon, 25 Feb 2008) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -324,7 +324,8 @@ class SpeechGenerator:
         #
         labels = self._script.findUnrelatedLabels(obj)
         for label in labels:
-            utterances.append(label.name)
+            name = self._getSpeechForObjectName(label)
+            utterances.extend(name)
 
         self._debugGenerator("_getSpeechForAlert",
                              obj,
@@ -1445,6 +1446,12 @@ class SpeechGenerator:
                 if speakAll:
                     for i in range(0, parent_table.nColumns):
                         cell = parent_table.getAccessibleAt(row, i)
+                        if not cell:
+                            debug.println(debug.LEVEL_WARNING,
+                                 "ERROR: speechgenerator." \
+                                 + "_getSpeechForTableCellRow" \
+                                 + " no accessible at (%d, %d)" % (row, i))
+                            continue
                         state = cell.getState()
                         showing = state.contains(pyatspi.STATE_SHOWING)
                         if showing:
